;********************************************************************
;*                                                                  *
;*        Morse coded ID message: adjust lines 700 ... 716          *
;*                                                                  *
;********************************************************************
;********************************************************************
;*      Kontroler in CW identifikator govornega repetitorja         *
;*                                                                  *
;*      Originalni PCB CW identifikatorja modificirati:             *
;*     (samo stari PCB V1; PCB V2 in PCB AS210115 brez modifikacije)*
;*                                                                  *
;*      - vezje za casovno omejitev oddaje (okoli T2) opusceno      *
;*      - RB5 direktno krmili bazo T3 prek R5 (PTT)                 *
;*                                                                  *
;*                                                                  *
;*      Frekvenca kvarca 8,000 MHz                                  *
;*                                                                  *
;*	Verzija 1.4, spremembe glede na V1.3:                       *
;*                                                                  *
;*      - prozenje oddaje identifikacije v primeru, da pred PTT     *
;*        ni aktivnosti najmanj 45 s (RPT v stanju ?akanja)         *
;*                                                                  *
;*	Verzija 1.3, spremembe glede na V1.2:                       *
;*                                                                  *
;*      - roger beep CW "E" ob prekinjajocem signalu na glavnem     *
;*        (primarnem) vhodu RX                                      *
;*                                                                  *
;*      Verzija 1.2 (s55usc), junij 2009                            *
;*                                                                  *
;*	Dodano glede na predhodne verzije:                          *
;*                                                                  *
;*      - drugi sql. det. vhod za crossconect povezavo s 6m RX      *
;*        (brez "roger beep" tona) (JP5)                            *    
;*                                                                  *
;*      - vhod za detekcijo prisotnosti napajanja iz omrezja in     *
;*        prozenje "roger bep" = CW "B" (JP6)                       *
;*                                                                  *
;*                                                                  *
;********************************************************************
;*                                                                  *
;*        Morse coded ID message: adjust lines 700 ... 716          *
;*                                                                  *
;********************************************************************

SUBTITL "CW beacon"
LIST     P=16F628A

#include "P16F628A.inc"     ; Iclude za definicijo Config bit

;********************************************************************
;*      Configuration bits settings                                 *
;********************************************************************
;
;		Oscillator			HS 8 MHz
;		WDT				On
;		Power up timer			Enabled
;		master clear enable		Enabled
;		Brown out detect		Enabled
;		Low voltage program		Disabled
;		Data EE read protect	        Disabled
;		Code protect			Off

; __config 0xFF75

 __CONFIG _FOSC_HS & _WDTE_ON & _PWRTE_ON & _MCLRE_ON & _BOREN_ON & _LVP_OFF & _CPD_OFF & _CP_OFF

 
;********************************************************************
;*      Sistemske  konstante                                        *
;********************************************************************
dit_lngth			equ	d'5'	; Dolzina trajanja CW pikice (T = n x 10ms)
dah_lngth			equ	d'15'	; Dolzina trajanja CW crtice (T = n x 10ms)
interditdah_lngth		equ	d'5'	; Dolzina trajanja pavze med dvema crticama ali pikicama (T = n x 10ms)
interchar_lngth			equ	d'35'	; Dolzina pavze med dvema CW znakoma (T = n x 10ms)

rec_on_dly			equ	d'5'	; Stevilo zaporednih odcitkov aktiviranega SQL za detekcijo sprejema (T = n x 10ms, priporocena vrednost 10)
rec_off_dly			equ	d'10'	; Stevilo zaporednih odcitkov neaktivnega SQL za detekcijo konca sprejema (T = n x 10ms, priporocena vrednost 10)

xcnct_on_dly			equ	d'5'	; Stevilo zaporednih odcitkov aktiviranega 6m SQL za detekcijo crossconect sprejema (T = n x 10ms, priporocena vrednost 10)
xcnct_off_dly			equ	d'10'	; Stevilo zaporednih odcitkov neaktivnega 6m SQL za detekcijo konca crossconect sprejema (T = n x 10ms, priporocena vrednost 10)

tx_timeout			equ	d'10'	; Cas do blokade oddaje pri neprekinjeni oddaji (T = n x 1min, priporocena vrednost je 10)

tx_tail_del			equ	d'12'	; Konstanta drzanja TX po koncu aktivnega sprejema oz. signala na vhodu v sprejemnik (T = n x 100ms, priporoceno 12)
cw_msg_tail_del			equ	d'15'	; Zakasnitev drzanja oddaje po koncu CW sporocila (T = n x 100ms, priporoceno 15)

timeout_death			equ	d'15'	; Cas blokade delovanja po izteku casovne omejitve oddaje (T = n x 1s, priporocena vrednost 15)

tm_to_beep			equ	d'9'	; Cas do "padca" TX, ko se odda roger beep (T = n x 100ms, priporocena vrednost 5)

xcnct_blck_tmo			equ	d'180'	; Cas do blokade zaradi predolgega neprekinjenega sprejema na 6m xcnct vhodu (T = n x 1s, priporocena vrednost 240)
xcnct_blck_durat		equ	d'60'	; Trajanje blokade zaradi predolgega neprekinjenega sprejema na 6m xcnct vhodu (T = n x 1s, priporocena vrednost 60)
	    
los_det_t_maks			equ	d'15'	; Maksimalna dolzina trajanja prekinitve, da jo se obravnavam kot LOS na vhodu primarnega RX (T = n x 100ms, priporoceno 15)
los_det_vld			equ	d'30'	; Veljavnost detektiranega stanja LOS na vhodu primarnega RX (T = n x 100ms, priporocena vrednost 30)
			
iden_no_sig_dly			equ	d'30'	; * * * Dodano 7.10.2021:  Trajanje (zakasnitev) do oddaje identifikacije zaradi odsotnosti aktivnosti RPT pred PTT
						;                         (T = n x 1 s, priporocena vrednost 30)
			
			
;********************************************************************
;*      Register Assignments                                        *
;********************************************************************
;*      Bank 0 00h...1Fh (rp1:rp0 ... 0:0)                          *
;********************************************************************
indir   equ	0x00		; Source/destination for indirect addressing.
tmr0	equ	0x01		; Timer 0 register
pcl     equ	0x02		; Program Counter - lower byte
status  equ	0x03		; Status Register
irp     equ     7
rp1     equ     6
rp0     equ     5       	; Register page select bit - 0
to      equ     4
pd      equ     3
zflag   equ     2       	; Zero Flag of Status Register.
dc      equ     1       	; Digit carry
carry   equ     0       	; Carry Flag of Status Register.

fsr     equ     0x04		; Indirect data memory address pointer
porta   equ     0x05		; Port A data register
portb   equ     0x06		; Port b data register

pclath  equ     0x0A		; Program counter latch - higher bits
intcon  equ     0x0B		; Interrupt control register
gie     equ     7		; bit 7 of intcon
peie    equ     6
t0ie    equ     5
inte    equ     4
rbie    equ     3
t0if    equ     2
intf    equ     1
rbif    equ     0		; Port B change interrupt bit (bit 0 of intcon)

pir1    equ     0x0C		;
rcif	equ	5		; UART RX flag
txif	equ	4		; UART TX interrupt flag
ccp1if	equ	2		; CCP module interrupt flag
tmr2if	equ	1		; Timer 2 interrupt flag

tmr1l   equ     0x0E		;
tmr1h   equ     0x0F		;
t1con   equ     0x10		;
tmr1on	equ	0		; TMR1 "on" bit

tmr2    equ     0x11		;
t2con   equ     0x12		;

ccpr1l  equ     0x15		;
ccpr1h  equ     0x16		;
ccp1con equ     0x17		;
rcsta   equ     0x18		; receive status and control register
cren	equ	4		; Continuous receive bit
ferr	equ	2		; Framing error bit
oerr	equ	1		; Overrun error bit

txreg   equ     0x19		; Transmit status and control register
rcreg   equ     0x1A		;	 

cmcon	equ     0x1F		;
				
;********************************************************************
;*      Bank 1 80h...9Fh (rp1:rp0 ... 0:1)                          *
;********************************************************************
opt_reg equ     0x01		; Option register
trisa   equ     0x05		; Port A data direction register
trisb   equ     0x06		; Port B data direction register
pie1    equ     0x0C		; Peripheral interrupt enable register 1

pcon    equ     0x0E		;
pr2     equ     0x12		;
txsta   equ     0x18		;
spbrg   equ     0x19		;

eedata  equ	0x1A		;
eeadr   equ	0x1B		;

eecon1  equ	0x1C		;
rd	equ	0		; RD bit registra eecon1
wr	equ	1		; WR bit registra eecon1
wren	equ	2		; Write enable bit registra eecon1

eecon2  equ	0x1D		;

vrcon	equ	0x9F


;********************************************************************
;*      Bank 2 100h...11Fh (rp1:rp0 ... 1:0)                        *
;********************************************************************
;********************************************************************
;*      Bank 3 180h...1EFh (rp1:rp0 ... 1:1)                        *
;********************************************************************

;********************************************************************
;*      Data locations in RAM.                                      *
;********************************************************************
;*      Data locations mapped in all four RAM pages = 70h ... 7Fh   *
;********************************************************************
stack_0			equ		0x70	; Simulirani stack
stack_1			equ		0x71	; Simulirani stack
stack_2			equ		0x72	; Simulirani stack
stack_3			equ		0x73	; Simulirani stack

int_uni_0		equ		0x74	; Univerzalna pomnilna lokacija, uporaba izkljucno v prekinitvenem podprogramu
int_uni_1		equ		0x75	; Univerzalna pomnilna lokacija, uporaba izkljucno v prekinitvenem podprogramu

;********************************************************************
;*      Spremenljivke RAM page 0 = 20h ... 6Fh                      *
;********************************************************************
uni_0			equ		0x20	; Univerzalna pomnilna lokacija 1
uni_1			equ		0x21	; Univerzalna pomnilna lokacija 2

loop_cnt		equ		0x22	; Stevec prehodov skozi programsko zanko
int_cnt			equ		0x23	; Stevec prekinitev nizke prioritete
sine_tone_cnt		equ		0x24	; Stevec generiranja sinusnega tona

ad_out			equ		0x25	; Vzorec, ki gre v D/A pretvornik (izhodni audio signal)

flag_0			equ		0x26	; Razne zastavice

rgr_beep		equ		7	; Zahteva za oddajo "roger beep" tona
cw_msg_ptt		equ		6	; Zahteva za vklop TX za oddajo CW sporocila
ptt_flg			equ		5	; Zastavica zahteve za oddajo (aktiviran PTT)
ptt_flg_old		equ		4	; Predhodna vrednost zastavice zahteve za oddajo (aktiviran PTT)
tx_iden_msg		equ		3	; Zastavica zahteve za takojsnjo oddajo CW identifikacijskega sporocila
iden_tout		equ		2	; Zastavica prozenja ident. sporocila v rezimu avtomatskega periodicnega prozenja
rec			equ		1	; Detektiran aktivni sprejem
cw_tone			equ		0	; Zastavica oddaje CW tona

flag_1			equ		0x27	; Razne zastavice

xcnct			equ		2	; Zastavica prozenja PTT 70cm TX zaradi detektiranega aktivnega sprejema na crossconect 6m RX
xcnct_sql		equ		1	; Detektiran aktivni sprejem na crossconect 6m RX
loop_trg		equ		0	; Zastavica prozenja izvajanja glavne programske zanke



msg_iden_pnt		equ		0x28	; Kazalec oddaje identifikacijskega sporocila
dit_dah_cnt 		equ		0x29	; Stevilo pikic in crtic trenutno oddajanega znaka, ki se cakajo na oddajo
cw_tone_cnt		equ		0x2A	; Stevec trajanja CW tona
cw_paus_cnt		equ		0x2B	; Stevec trajanja CW pavze

rec_on_cnt		equ		0x2C	; Stevec aktivnih odcitkov sql za aktivni sprejem
rec_off_cnt		equ		0x2D	; Stevec neaktivnih odcitkov sql za konec neaktivnega sprejema

ss_cnt			equ		0x2E	; Stevec stotink sekunde
ds_cnt			equ		0x2F	; Stevec desetink sekunde za prozenje minutnih akcij
sec_cnt			equ		0x30	; Stevec za prozenje sekundnih akcij

iden_rec_blk_del	equ		0x31	; Zakasnitev prozenja odddaje CW identifikacije po koncu aktivnega sprejema
iden_rec_blk_to		equ		0x32	; Stevec zakasnitve prozenja odddaje CW identifikacije po koncu aktivnega sprejema

iden_per_cnt_l		equ		0x33	; Stevec periode oddaje identifikacijskega sporocila, nizji byte
iden_per_cnt_h		equ		0x34	; Stevec periode oddaje identifikacijskega sporocila, visji byte

tx_iden_per_l		equ		0x35	; Perioda oddaje identifikacije, nizji byte
tx_iden_per_h		equ		0x36	; Visji byte, vrednost dolocena s polozajem DIP stikalc SW3 in SW4

tx_tmout_cntr		equ		0x37	; Stevec time-out oddaje

tmo_blk_tmr		equ		0x38	; Stevec blokade po preteku casovne omejitve oddaje
tx_tail_cntr		equ		0x39	; Stevec drzanja TX po koncu zadnje zahteve za oddajo
rgr_beep_cntr		equ		0x3A	; Stevec trajanja oddaje "roger beep" tona

xcnct_on_cnt		equ		0x3B	; Stevec aktivnih odcitkov crossconect sql det. za aktivni sprejem
xcnct_off_cnt		equ		0x3C	; Stevec neaktivnih odcitkov crossconect sql det. za konec neaktivnega sprejema

xcnct_blck_cnt		equ		0x3D	; Stevec vzdrzevanja blokade PTT zaradi sprejema na crossconnect povezavi 6m vhoda
xcnct_onair_cnt		equ		0x3E	; Stevec, ki odsteva dovoljeni cas trajanja aktivnega xconnect repetiranja

startup_dly_cnt		equ		0x3F	; Stevec zakasnitve ob zagonu

los_rx_tmr		equ		0x40	; Timer detekcije stanja kratkotrajne izgube signala na vhodu primarnega RX
los_beep_tmr		equ		0x41	; Timer veljavnosti stanja LOS za prozenje roger beep CW "E"

iden_no_sig_to		equ		0x42	; * * * Dodano 7.10.2021 - Stevec oddaje identifikacije zaradi odsotnosti aktivnosti RPT pred PTT


;********************************************************************
;*      Spremenljivke RAM page 1 = A0h ... EFh                      *
;********************************************************************

;********************************************************************
;*      Spremenljivke RAM page 2 = 120h ... 14Fh                    *
;********************************************************************

;********************************************************************
;*      Peripheral devices                                          *
;********************************************************************
; Port A
dip_sw_1		equ		0	; Vhod - DIP stikalce 1
dip_sw_2		equ		1	; Vhod - DIP stikalce 2

sql_det			equ		4	; SQL detect, visoko stanje - aktiven sprejem



; Port B
ptt			equ		5	; Izhod za prozenje PTT (aktivno stanje je 5V)
jp1			equ		6	; Vhod - JP1, izrabljen za crossconect SQL detect
jp2			equ		7	; Vhod - JP2, izrabljen za detekcijo prisotnosti napajanja iz omrezja

;********************************************************************
;*      Reset vectors                                               *
;********************************************************************
	org     0x0000
	goto    reset           		; Reset vector

	org     0x0004          		; Interrupt vector

;********************************************************************
;********************************************************************
;*                                                                  *
;*      Interrupt subrutine                                         *
;*      - Interrupt management, approx. 15 clock cycles             *
;*                                                                  *
;********************************************************************
;********************************************************************
	movwf   stack_0         		; Push W to stack
	swapf   status,w        		;                               
	movwf   stack_1         		; Push status word to stack
	movf    fsr,w
	movwf   stack_2         		; Push FSR to stack
	movf    pclath,w
	movwf   stack_3         		; Push PCLATH to stack

	clrf    status          		; Select memory page 0
	clrf    pclath

	btfsc   pir1,tmr2if     		; Periodicna prekinitev timer 2 modula?
	goto    int_tmr2        		; Da, servisiraj periodicno prekinitev tmr2
int_reset
	goto    int_reset       		; Huda napaka v izvajanju programa. Pocakaj na WDT reset.

;********************************************************************
;*                                                                  *
;*      Periodicna prekinitev timerskega modula 2, T = 62,5us       *
;*                                                                  *
;********************************************************************
int_tmr2
	clrf    status				; Preklop na RAM/reg. stran 0
	bcf		pir1,tmr2if		; Clear timer 2 interrupt flag

	movlw	0x10
	movwf	ad_out				; Privzeta vrednost D/A konverterja je nevtralna (obseg D/A: 0x00 .. 0x1F)

;--------------------------------------------------------------------
;*      Vejitve na posamezne akcije                                 *
;--------------------------------------------------------------------
	btfsc	flag_0,cw_tone			; Zahteva za oddajo CW tona? Ce ni, preskoci.
	goto	int_tmr0_sine_tone		; Oddaja CW tona v teku
	movf	sine_tone_cnt,f			; Stevec oddaje sinusnega tona
	btfss	status,zflag			; Stevec na nicli, tudi rep sinusa CW tona se vec ne oddaja
	goto	int_tmr0_sine_tail		; Oddaja repa tona se vedno traja (oddaja upadajocega sinusa do 0V)

	goto	int_tmr0_ok			; Voice recorder neaktiven

;--------------------------------------------------------------------
;*                                                                  *
;*      Generiranje sinusnega tona 800Hz z glavo dolzine 90 vzorcev.*
;*                                                                  *
;--------------------------------------------------------------------
int_tmr0_sine_tone
	movf	sine_tone_cnt,w			; Stevec oddaje sinusnega tona
	call	sine_tab			; Precitam iz tabele pripadajoci vzorec sinusa
	movwf	ad_out				; Tocko sinusa dam na izhod (D/A)

	incf	sine_tone_cnt,w			; Povecam stevec oddaje sinusnega tona
	movwf	sine_tone_cnt			; Shranim v register

	sublw	d'110'				; Stevec CW tona primerjam z vrednostjo za 1 vecjo od zadnjega vzorca sinusa
	btfss	status,zflag			; Stevec dosegel zadnji vzorec sinusa, preskoci
	goto	int_tmr0_sine_tone_s1		; Stevec se ni dosegel zadnjega vzorca sinusa, koncaj s tem delom podprograma
	movlw	d'90'
	movwf	sine_tone_cnt			; Stevec postavim nazaj na zacetek osnovnega sinusa
int_tmr0_sine_tone_s1
	goto	int_tmr0_ok

;--------------------------------------------------------------------
;*                                                                  *
;*      Generiranje repa sinusnega tona 800Hz dolzine 90 vzorcev.   *
;*                                                                  *
;--------------------------------------------------------------------
int_tmr0_sine_tail
	movf	sine_tone_cnt,w			; Stevec oddaje sinusnega tona
	call	sine_tab			; Precitam iz tabele pripadajoci vzorec sinusa
	movwf	ad_out				; Tocko sinusa dam na izhod (D/A)

	incf	sine_tone_cnt,w			; Povecam stevec oddaje sinusnega tona
	movwf	sine_tone_cnt			; Shranim v register

	sublw	d'201'				; Stevec sinusnega tona primerjam z vrednostjo za 1 vecjo od zadnjega vzorca repa sinusa
	btfsc	status,zflag			; Stevec se ni dosegel zadnjega vzorca repa sinusa, preskoci
	clrf	sine_tone_cnt			; Stevec resetiram
	goto	int_tmr0_ok

;*------------------------------------------------------------------*
;*      Konec akcij timerskega modula 0                             *
;*------------------------------------------------------------------*
int_tmr0_ok
	goto    int_ok

;--------------------------------------------------------------------
;*                                                                  *
;*      Del kode, ki se izvrsi ob vsaki prekinitvi                  *
;*                                                                  *
;--------------------------------------------------------------------
int_ok
	decfsz	int_cnt,f		; Zmanjsam stevec prozenja prekinitve (tmr2)
	goto	int_cnt_nz		; Stevec se ni dosegel vrednosti 0
	movlw	d'160'			; Stevec inicializiram na 160 prehodov skozi prek. podprogram za ponovno prozenje izvajanja glavne zanke
	movwf	int_cnt
	bsf	flag_1,loop_trg		; Oznacim z zastavico, da je napocil cas za izvajanje glavne programske zanke
int_cnt_nz

	btfss	portb,ptt		; Stanje na PTT izhodu visoko --> preskoci
	bcf	ad_out,ptt		; Ustrezno postavim bit, ki gre na portb/PTT
	btfsc	portb,ptt		; Stanje na PTT izhodu nizko --> preskoci
	bsf	ad_out,ptt		; Ustrezno postavim bit, ki gre na portb/PTT

	bcf	ad_out,jp1		; JP1 in JP2 sta vhoda, zato ta dva ukaza nimata posebnega pomena
	bcf	ad_out,jp2

	movf	ad_out,w		; Vrednost novega vzorca za D/A pretvornik ter stanje PTT ...
	movwf	portb			; ... dam na port B

;*------------------------------------------------------------------*
;*      Izstop iz prekinitvenega podprograma                        *
;*------------------------------------------------------------------*
	movf    stack_3,w       ; Pick PCLATH from stack
	movwf   pclath
	movf    stack_2,w       ; Pick FSR from stack
	movwf   fsr
	swapf   stack_1,w       ; Pick STATUS from stack
	movwf   status          ; Move w into STATUS register
	swapf   stack_0,f       ; Swap stack location containing original value of w
	swapf   stack_0,w       ; Swap stack location containing original value of w into w
	retfie

;********************************************************************
;*      Tabela z definiranimi tockami sinusa 800Hz                  *
;*		Tabela je dolga ca. 160 byte in je znotraj drugega bloka    *
;*      programskega spomina v izogib tezavam s PCLATH.             *
;********************************************************************
	org	0x0100			; Tabela shranjena na fiksni lokaciji znotraj drugega bloka 256 byte programskega spomina
sine_tab
	movwf	fsr			; Zacasno shranim
	movlw	0x01
	movwf   pclath			; Inicializiram PCLATH za skoke po tabeli
	movf	fsr,w
	addwf   pcl,f

	retlw	d'16'			;  0. vzorec: 1. tocka glave sinusa (polperioda 0,1 * A)
	retlw	d'17'			;  1. vzorec
	retlw	d'17'			;  2. vzorec
	retlw	d'17'			;  3. vzorec
	retlw	d'18'			;  4. vzorec
	retlw	d'18'			;  5. vzorec
	retlw	d'18'			;  6. vzorec
	retlw	d'17'			;  7. vzorec
	retlw	d'17'			;  8. vzorec
	retlw	d'17'			;  9. vzorec

	retlw	d'16'			; 10. vzorec: polperioda -0,2 * A
	retlw	d'15'			; 11. vzorec
	retlw	d'14'			; 12. vzorec
	retlw	d'14'			; 13. vzorec
	retlw	d'13'			; 14. vzorec
	retlw	d'13'			; 15. vzorec
	retlw	d'13'			; 16. vzorec
	retlw	d'14'			; 17. vzorec
	retlw	d'14'			; 18. vzorec
	retlw	d'15'			; 19. vzorec

	retlw	d'16'			; 20. vzorec: polperioda 0,3 * A
	retlw	d'18'			; 21. vzorec
	retlw	d'19'			; 22. vzorec
	retlw	d'20'			; 23. vzorec
	retlw	d'21'			; 24. vzorec
	retlw	d'21'			; 25. vzorec
	retlw	d'21'			; 26. vzorec
	retlw	d'20'			; 27. vzorec
	retlw	d'19'			; 28. vzorec
	retlw	d'18'			; 29. vzorec

	retlw	d'16'			; 30. vzorec: polperioda -0,4 * A
	retlw	d'14'			; 31. vzorec
	retlw	d'12'			; 32. vzorec
	retlw	d'11'			; 33. vzorec
	retlw	d'10'			; 34. vzorec
	retlw	d'10'			; 35. vzorec
	retlw	d'10'			; 36. vzorec
	retlw	d'11'			; 37. vzorec
	retlw	d'12'			; 38. vzorec
	retlw	d'14'			; 39. vzorec

	retlw	d'16'			; 40. vzorec: polperioda 0,5 * A
	retlw	d'18'			; 41. vzorec
	retlw	d'20'			; 42. vzorec
	retlw	d'22'			; 43. vzorec
	retlw	d'23'			; 44. vzorec
	retlw	d'24'			; 45. vzorec
	retlw	d'23'			; 46. vzorec
	retlw	d'22'			; 47. vzorec
	retlw	d'20'			; 48. vzorec
	retlw	d'18'			; 49. vzorec

	retlw	d'16'			; 50. vzorec: polperioda -0,6 * A
	retlw	d'13'			; 51. vzorec
	retlw	d'11'			; 52. vzorec
	retlw	d'9'			; 53. vzorec
	retlw	d'7'			; 54. vzorec
	retlw	d'7'			; 55. vzorec
	retlw	d'7'			; 56. vzorec
	retlw	d'9'			; 57. vzorec
	retlw	d'11'			; 58. vzorec
	retlw	d'13'			; 59. vzorec

	retlw	d'16'			; 60. vzorec: polperioda 0,7 * A
	retlw	d'19'			; 61. vzorec
	retlw	d'22'			; 62. vzorec
	retlw	d'24'			; 63. vzorec
	retlw	d'26'			; 64. vzorec
	retlw	d'27'			; 65. vzorec
	retlw	d'26'			; 66. vzorec
	retlw	d'24'			; 67. vzorec
	retlw	d'22'			; 68. vzorec
	retlw	d'19'			; 69. vzorec

	retlw	d'16'			; 70. vzorec: polperioda -0,8 * A
	retlw	d'12'			; 71. vzorec
	retlw	d'9'			; 72. vzorec
	retlw	d'6'			; 73. vzorec
	retlw	d'5'			; 74. vzorec
	retlw	d'4'			; 75. vzorec
	retlw	d'5'			; 76. vzorec
	retlw	d'6'			; 77. vzorec
	retlw	d'9'			; 78. vzorec
	retlw	d'12'			; 79. vzorec

	retlw	d'16'			; 80. vzorec: polperioda 0,9 * A
	retlw	d'20'			; 81. vzorec
	retlw	d'24'			; 82. vzorec
	retlw	d'27'			; 83. vzorec
	retlw	d'29'			; 84. vzorec
	retlw	d'30'			; 85. vzorec
	retlw	d'29'			; 86. vzorec
	retlw	d'27'			; 87. vzorec
	retlw	d'24'			; 88. vzorec
	retlw	d'20'			; 89. vzorec

	retlw	d'16'			; 90. vzorec: 1. tocka sinusa osnovnega
	retlw	d'11'			; 91. vzorec
	retlw	d'7'			; 92. vzorec
	retlw	d'4'			; 93. vzorec
	retlw	d'2'			; 94. vzorec
	retlw	d'1'			; 95. vzorec
	retlw	d'2'			; 96. vzorec
	retlw	d'4'			; 97. vzorec
	retlw	d'7'			; 98. vzorec
	retlw	d'11'			; 99. vzorec
	retlw	d'16'			; 100. vzorec
	retlw	d'21'			; 101. vzorec
	retlw	d'25'			; 102. vzorec
	retlw	d'28'			; 103. vzorec
	retlw	d'30'			; 104. vzorec
	retlw	d'31'			; 105. vzorec
	retlw	d'30'			; 106. vzorec
	retlw	d'28'			; 107. vzorec
	retlw	d'25'			; 108. vzorec
	retlw	d'21'			; 109. vzorec

	retlw	d'16'			; 110. vzorec: (polperioda -0,9 * A), 1. tocka repa sinusa (rep sinusa =  glava sinusa v obratnem vrstnem redu)
	retlw	d'12'			; 111. vzorec
	retlw	d'8'			; 112. vzorec
	retlw	d'5'			; 113. vzorec
	retlw	d'3'			; 114. vzorec
	retlw	d'2'			; 115. vzorec
	retlw	d'3'			; 116. vzorec
	retlw	d'4'			; 117. vzorec
	retlw	d'8'			; 118. vzorec
	retlw	d'12'			; 119. vzorec

	retlw	d'16'			; 120. vzorec: polperioda +0,8 * A
	retlw	d'20'			; 121. vzorec
	retlw	d'23'			; 122. vzorec
	retlw	d'26'			; 123. vzorec
	retlw	d'27'			; 124. vzorec
	retlw	d'28'			; 125. vzorec
	retlw	d'27'			; 126. vzorec
	retlw	d'26'			; 127. vzorec
	retlw	d'23'			; 128. vzorec
	retlw	d'20'			; 129. vzorec

	retlw	d'16'			; 130. vzorec: polperioda -0,7 * A
	retlw	d'13'			; 131. vzorec
	retlw	d'10'			; 132. vzorec
	retlw	d'8'			; 133. vzorec
	retlw	d'6'			; 134. vzorec
	retlw	d'5'			; 135. vzorec
	retlw	d'6'			; 136. vzorec
	retlw	d'8'			; 137. vzorec
	retlw	d'10'			; 138. vzorec
	retlw	d'13'			; 139. vzorec

	retlw	d'16'			; 140. vzorec: polperioda 0,6 * A
	retlw	d'19'			; 141. vzorec
	retlw	d'21'			; 142. vzorec
	retlw	d'23'			; 143. vzorec
	retlw	d'27'			; 144. vzorec
	retlw	d'27'			; 145. vzorec
	retlw	d'27'			; 146. vzorec
	retlw	d'23'			; 147. vzorec
	retlw	d'21'			; 148. vzorec
	retlw	d'19'			; 149. vzorec

	retlw	d'16'			; 150. vzorec: polperioda -0,5 * A
	retlw	d'14'			; 151. vzorec
	retlw	d'12'			; 152. vzorec
	retlw	d'10'			; 153. vzorec
	retlw	d'9'			; 154. vzorec
	retlw	d'8'			; 155. vzorec
	retlw	d'9'			; 156. vzorec
	retlw	d'10'			; 157. vzorec
	retlw	d'12'			; 158. vzorec
	retlw	d'14'			; 159. vzorec

	retlw	d'16'			; 160. vzorec: polperioda 0,4 * A
	retlw	d'18'			; 161. vzorec
	retlw	d'20'			; 162. vzorec
	retlw	d'21'			; 163. vzorec
	retlw	d'22'			; 164. vzorec
	retlw	d'22'			; 165. vzorec
	retlw	d'22'			; 166. vzorec
	retlw	d'21'			; 167. vzorec
	retlw	d'20'			; 168. vzorec
	retlw	d'18'			; 169. vzorec

	retlw	d'16'			; 170. vzorec: polperioda -0,3 * A
	retlw	d'14'			; 171. vzorec
	retlw	d'13'			; 172. vzorec
	retlw	d'12'			; 173. vzorec
	retlw	d'11'			; 174. vzorec
	retlw	d'11'			; 175. vzorec
	retlw	d'11'			; 176. vzorec
	retlw	d'12'			; 177. vzorec
	retlw	d'13'			; 178. vzorec
	retlw	d'14'			; 179. vzorec

	retlw	d'16'			; 180. vzorec: polperioda 0,2 * A
	retlw	d'17'			; 181. vzorec
	retlw	d'18'			; 182. vzorec
	retlw	d'18'			; 183. vzorec
	retlw	d'19'			; 184. vzorec
	retlw	d'19'			; 185. vzorec
	retlw	d'19'			; 186. vzorec
	retlw	d'18'			; 187. vzorec
	retlw	d'18'			; 188. vzorec
	retlw	d'17'			; 189. vzorec

	retlw	d'16'			; 190. vzorec: polperioda -0,1 * A
	retlw	d'15'			; 191. vzorec
	retlw	d'15'			; 192. vzorec
	retlw	d'15'			; 193. vzorec
	retlw	d'14'			; 194. vzorec
	retlw	d'14'			; 195. vzorec
	retlw	d'14'			; 196. vzorec
	retlw	d'15'			; 197. vzorec
	retlw	d'15'			; 198. vzorec
	retlw	d'15'			; 199. vzorec

	retlw	d'16'			; 200. vzorec: koncni nevtralni vzorec
	retlw	d'16'			; 
	retlw	d'16'			; 
	retlw	d'16'			; 

;********************************************************************
;*      Morse coded ID message                                      *
;********************************************************************
	org	0x0200			; Tabela shranjena na fiksni lokaciji znotraj tretjega bloka 256 byte programskega spomina
iden_msg_tab
	andlw	0x1F			; Definirane kode 0x00 ... 0x1F
	movwf	fsr			; Zacasno shranim
	movlw	0x02
	movwf   pclath			; Inicializiram PCLATH za skoke po tabeli
	movf	fsr,w
	addwf   pcl,f
	
	retlw   d'30'			; Stevilo znakov sporocila
	retlw   b'00110000'		; T; 1, -
	retlw   b'01000000'		; I; 2, ..
	retlw   b'01011000'		; M; 2, --
	retlw   b'00100000'		; E; 1, .
	retlw   b'00000000'		; Pavza; 0
	retlw   b'01111100'		; O; 3, ---
	retlw   b'01100100'		; U; 3, ..-
	retlw   b'00110000'		; T; 1, -
	retlw   b'00000000'		; Pavza; 0
	retlw   b'00000000'		; Pavza; 0
	retlw   b'01110000'		; D; 3, -..
	retlw   b'00100000'		; E; 1, .
	retlw   b'00000000'		; Pavza; 0
	retlw   b'10000100'		; F; 4, ..-.
	retlw   b'01011000'		; M; 2, --
	retlw   b'00000000'		; Pavza; 0
	retlw   b'01101000'		; R; 3, .-.
	retlw   b'10001100'		; P; 4, .--.
	retlw   b'00110000'		; T; 1, -
	retlw   b'00000000'		; Pavza; 0
	retlw   b'00000000'		; Pavza; 0
	retlw   b'10011010'		; Q; 4, --.-
	retlw   b'00110000'		; T; 1, -
	retlw   b'10000000'		; H; 4, ....
	retlw   b'10001000'		; L; 4, .-..
	retlw   b'01111100'		; O; 3, ---
	retlw   b'10010100'		; C; 3, -.-.
	retlw   b'00000000'		; Pavza; 0
	retlw   b'00000000'		; Pavza; 0
	retlw   b'01110100'		; K; 3, -.-
	retlw   b'00000000'		; Pavza; 0
	retlw   b'00000000'		; Pavza; 0
;--------------------------------------------------------------------
;*      Codes of the morse coded characters                         *
;--------------------------------------------------------------------
;	retlw   b'01001000'		; A; 2, .-
;	retlw   b'10010000'		; B; 4, -...
;	retlw   b'10010100'		; C; 4, -.-.
;	retlw   b'01110000'		; D; 3, -..
;	retlw   b'00100000'		; E; 1, .
;	retlw   b'10000100'		; F; 4, ..-.
;	retlw   b'01111000'		; G; 3, --.
;	retlw   b'10000000'		; H; 4, ....
;	retlw   b'01000000'		; I; 2, ..
;	retlw   b'10001110'		; J; 4, .---
;	retlw   b'01110100'		; K; 3, -.-
;	retlw   b'10001000'		; L; 4, .-..
;	retlw   b'01011000'		; M; 2, --
;	retlw   b'01010000'		; N; 2, -.
;	retlw   b'01111100'		; O; 3, ---
;	retlw   b'10001100'		; P; 4, .--.
;	retlw   b'10011010'		; Q; 4, --.-
;	retlw   b'01101000'		; R; 3, .-.
;	retlw   b'01100000'		; S; 3, ...
;	retlw   b'00110000'		; T; 1, -
;	retlw   b'01100100'		; U; 3, ..-
;	retlw   b'01101100'		; W; 3, .--
;	retlw   b'10000010'		; V; 4, ...-
;	retlw   b'10010010'		; X; 4, -..-
;	retlw   b'10010110'		; y; 4, -.--
;	retlw   b'10011000'		; Z; 4, --..
;	retlw   b'10111111'		; 0; 5, -----
;	retlw   b'10101111'		; 1; 5, .----
;	retlw   b'10100111'		; 2; 5, ..---
;	retlw   b'10100011'		; 3; 5, ...--
;	retlw   b'10100001'		; 4; 5, ....-
;	retlw   b'10100000'		; 5; 5, .....
;	retlw   b'10110000'		; 6; 5, -....
;	retlw   b'10111000'		; 7; 5, --...
;	retlw   b'10111100'		; 8; 5, ---..
;	retlw   b'10111110'		; 9; 5, ----.
;	retlw   b'10110001'		; =; 5, -...-
;	retlw   b'00000000'		; SPACE

;--------------------------------------------------------------------
;*      DO NOT MODIFY THE FOLLOWING MESSAGE ! ! !                   *
;--------------------------------------------------------------------
;********************************************************************
;*      Tabela s sporocilom o preteku casovne omejitve oddaje       *
;********************************************************************
	retlw   b'10110001'		; =; 5, -...-
	retlw   b'00000000'		; Pavza; 0
	retlw   b'00110000'		; T; 1, -
	retlw   b'01000000'		; I; 2, ..
	retlw   b'01011000'		; M; 2, --
	retlw   b'00100000'		; E; 1, .
	retlw   b'00000000'		; Pavza; 0
	retlw   b'01101110'		; O; 3, ---
	retlw   b'01100100'		; U; 4, ..-
	retlw   b'00110000'		; T; 1, -
	retlw   b'00000000'		; Pavza; 0

;********************************************************************
;*      Inicializcija ob resetu                                     *
;********************************************************************
reset
	clrf    status          ; Lowest register banks selected
	clrf    pclath
	clrf    intcon

	clrwdt

	movlw   b'00000000'     ; Inicializacija port A, vsi vhodi
	movwf   porta
	movlw   b'11010000'     ; Inicializacija port B, PTT neaktiven, D/A izhod nevtralen (0x10)
	movwf   portb
	
	movlw	b'00000111'
	movwf	cmcon		; Izklop komparatorjev

	bsf     status,rp0      ; Dostop do banke registrov 1
	
	movlw   b'01001111'	; Initialisation of OPTION register
	;              ^^^------- WDT prescaler rate 1:128
	;             ^---------- Prescaler assigned to the WDT
	;           ^^----------- Timer 0 source: Low to hi transition of internal instruction cycle clock
	;          ^------------- Interrupt on rising edge of RB0 pin
	;         ^-------------- Port B pull-ups ENabled
	movwf   opt_reg          
 	
	movlw   b'11111111'     ; Konfiguracija port A, vsi vhodi
	movwf   trisa
	movlw   b'11000000'     ; Inicializacija port B, 2 x JMP vhod, PTT izhod, 5 x D/A izhod
	movwf   trisb
	bcf     status,rp0      ; Dostop do banke registrov 0

;--------------------------------------------------------------------
;*      Timer 1 - neizkoriscen                                      *
;--------------------------------------------------------------------
	clrf	status		; Dostop do registrske banke 0
	clrf	tmr1l		; Reset 16 bitnega stevca timerskega amodula 1
	clrf	tmr1h
	movlw	b'00110000'	; 1:8 prescale, timer 1 osc. off, internal clock, timer 1 off
	movwf	t1con

;--------------------------------------------------------------------
;*      Nastavitve timer 2 - periodicna prekinitev, T = 62,5 us     *
;*		pri f(Q) = 8  MHz                                           *
;--------------------------------------------------------------------
	clrf    status
	bsf     status,rp0	; Dostop do banke registrov 1
	movlw	d'124'		; 124
	movwf	pr2		; Interrupt on every 125 clock cycles (counts 0..124)
	bcf     status,rp0	; Dostop do banke registrov 0
	movlw	b'00000100'	; Timer 2 on, postscaler & prescaler set to 1:1 & 1:1
	movwf	t2con

;--------------------------------------------------------------------
;*      Nastavitve UART - 19200 bps 8N1 - V TEJ APLIKAC. IZKLJUCEN  *
;--------------------------------------------------------------------
	clrf    status
	bsf     status,rp0	; Dostop do banke registrov 1
	movlw   d'51'		; Baud rate 19230 bit/s pri f(Q) = 16,000 MHz (brgh = txsta<2> = 1 ... high speed)
				; Baud rate  9615 bit/s pri f(Q) =  8,000 MHz
	movwf   spbrg
	movlw   b'00000100'	; Don't care, 8 bit, TX DISabled, async, -, brgh=1, -, 9. bit ni pomemben
	movwf   txsta		; UART TX status and control register
	bcf     status,rp0; 	; Dostop do banke registrov 0
	movlw   b'00010000'	; Ser. port DISabled, 8-bit RX, -, RX en., no address detection, ostali 3 biti samo za branje
	movwf   rcsta		; UART receive status and control register

;--------------------------------------------------------------------
;*		Inicializacija spremenljivk, RAM page 0                     *
;--------------------------------------------------------------------
	clrf	status		; Preklop na RAM/reg str. 0
	
	clrf	loop_cnt	; Stevec prehodov skozi programsko zanko
	clrf	sine_tone_cnt	; Stevec generiranja sinusnega tona
	clrf	flag_0		; Razne zastavice
	clrf	flag_1		; Razne zastavice

	clrf	msg_iden_pnt 	; Kazalec oddaje identifikacijskega sporocila
	clrf	dit_dah_cnt	; Stevilo pikic in crtic trenutno oddajanega znaka, ki se cakajo na oddajo
	clrf	cw_tone_cnt 	; Stevec trajanja CW tona
	clrf	cw_paus_cnt 	; Stevec trajanja CW pavze

	clrf	rec_on_cnt	; Stevec aktivnih odcitkov sql za aktivni sprejem
	clrf	rec_off_cnt	; Stevec neaktivnih odcitkov sql za konec neaktivnega sprejema

	clrf	xcnct_on_cnt	; Stevec aktivnih odcitkov crossconect sql det. za aktivni sprejem
	clrf	xcnct_off_cnt	; Stevec neaktivnih odcitkov crossconect sql det. za konec neaktivnega sprejema

	clrf	iden_rec_blk_to	; Stevec zakasnitve prozenja odddaje CW identifikacije po koncu aktivnega sprejema

	clrf	tx_tmout_cntr	; Stevec time-out oddaje

	clrf	tmo_blk_tmr	; Stevec blokade po preteku casovne omejitve oddaje
	clrf	tx_tail_cntr	; Stevec drzanja TX po koncu zadnje zahteve za oddajo

	clrf	rgr_beep_cntr	; Stevec trajanja oddaje "roger beep" tona

	clrf	xcnct_blck_cnt	; Stevec vzdrzevanja blokade PTT zaradi sprejema na crossconnect povezavi 6m vhoda
	clrf	xcnct_onair_cnt	; Stevec, ki odsteva dovoljeni cas trajanja aktivnega xconnect repetiranja

	clrf	los_rx_tmr	; Timer detekcije stanja kratkotrajne izgube signala na vhodu primarnega RX
	clrf	los_beep_tmr	; Timer veljavnosti stanja LOS za prozenje roger beep CW "E"

	movlw	d'160'		; Stevec prekinitev inicializiram na 160 prehodov skozi prek. podprogram za ponovno prozenje izvajanja glavne zanke
	movwf	int_cnt

	movlw	d'10'		; Desetinka sekunde je dolga 10 prehodov skozi glavno zanko
	movwf	ss_cnt		; Stevec stotink sekunde
	movlw	d'10'		; 10 desetink sekunde za prozenje sekundnih akcij
	movwf	ds_cnt		; Stevec desetink sekunde za prozenje sekundnih akcij
	movlw	d'60'
	movwf	sec_cnt		; Stevec za prozenje sekundnih akcij

	movlw	d'150'		; 150 * 1/10s zakasnitve, kasneje se v glavni zanki vrednost nastavi glede na stanje kodirnih stikalc
	movwf	iden_rec_blk_del ; Zakasnitev prozenja odddaje CW identifikacije po koncu aktivnega sprejema

	movlw	d'30'		; Prva oddaja ident. sporocila 30s po resetu
	movwf	iden_per_cnt_l	; Stevec periode oddaje identifikacijskega sporocila, nizji byte
	clrf	iden_per_cnt_h	; Stevec periode oddaje identifikacijskega sporocila, visji byte

	movlw	d'50'		; 50 x 10ms ..
	movwf	startup_dly_cnt	; ..dam v stevec zakasnitve ob zagonu programa

	movlw	iden_no_sig_dly	; * * * Dodano 7.10.2021 -  Trajanje (zakasnitev) do oddaje identifikacije zaradi odsotnosti aktivnosti RPT pred PTT ..
	movwf	iden_no_sig_to	; * * * Dodano 7.10.2021 -  .. dam v stevec oddaje identifikacije zaradi odsotnosti aktivnosti RPT pred PTT


;--------------------------------------------------------------------
;*      Prekinitvene nastavitve                                     *
;--------------------------------------------------------------------
	clrf	status		; Dostop do registerske banke 0
	clrf	pir1		; Vse zastavice perifernih prekinitev resetirane
	bsf	status,rp0	; Preklopim na dostop do registrske banke 1
	movlw   b'00000010'	; Timer 2 interrupt omogocen
	movwf   pie1
	clrf	status		; Preklop na dostop do registrske banke 0
	movlw   b'11000000'	; GIE in periferne prekinitve omogocene
	movwf   intcon

;--------------------------------------------------------------------
;*      Glavna programska zanka                                     *
;--------------------------------------------------------------------
zanka
	clrf	status		    ; Dostop do registrske banke 0
	clrf	pclath		    ; Vse akcije znotraj prve strani programskega spomina
	btfss	flag_1,loop_trg	    ; Preverim, ali je prekinitvena rutina postavila zastavico za prozenje casovno odvosnih akcij (10ms). Ce je, preskoci.
	goto	zanka		    ; 10ms se ni preteklo, cakam na zastavico.

	bcf	flag_1,loop_trg	    ; Zastavico brisem
	clrwdt			    ; Resetiram kuzapazi casovnik
	incf	loop_cnt,f	    ; Povecam stevec zanke

;--------------------------------------------------------------------
;*      Del glavne zanke, ki se izvede vsakih 10ms (100Hz)          *
;--------------------------------------------------------------------
	movf	startup_dly_cnt,f	; Stevec zakasnitve ob zagonu
	btfsc	status,zflag		; Ce stevec se ni prestel do nicle, preskoci naslednji ukaz in ukrepaj
	goto	zan_startup_del_ok	; Zakasnitev ob zagonu programa ni vec v akciji
	decf	startup_dly_cnt,f	; Stevec se ni prestel do 0, zakasnitev ob zagonu aktivna, zmanjsam stevec za 1
	goto	zanka			; Dokler traja zakasnitev ne izvajam nobenih akcij
zan_startup_del_ok

;--------------------------------------------------------------------
;*      Krmiljenje zastavice zahteve za oddajo (flag_0,ptt_flg)     *
;--------------------------------------------------------------------
	clrf	status			; Dostop do registrske banke 0

	btfsc	flag_0,ptt_flg		; Ce trenutna vrednost zastavice spuscena, preskoci..
	bsf	flag_0,ptt_flg_old	; ..sicer dvigni "predhodno" vrednost zastavice
	btfss	flag_0,ptt_flg		; Ce trenutna vrednost zastavice dvignjena, preskoci..
	bcf	flag_0,ptt_flg_old	; ..sicer spusti "predhodno" vrednost zastavice

	btfsc	flag_0,ptt_flg		; Ce zahteva za oddajo ni aktivna, preskoci
	goto	zan_ptt_flg_act		; Zahteva za oddajo aktivna

	btfsc	flag_0,cw_msg_ptt	; Zahteva za prozenje oddaje - CW sporocilo, ce ni, preskoci
	goto	zan_ptt_flg_dwn_s1	; Da, ukrepaj
	btfsc	flag_0,rec		; Detekcija aktivnega sprejema, ce ni, preskoci
	goto	zan_ptt_flg_dwn_s2	; Da, ukrepaj
	btfsc	flag_1,xcnct		; Detekcija aktivnega sprejema na 6m crossconnect RX, ce ni, preskoci
	goto	zan_ptt_flg_dwn_s3	; Da, ukrepaj
	goto	zan_ptt_flg_dwn_ok	; Ni zahtev za vklop TX

zan_ptt_flg_dwn_s1
	movlw	d'2'			; Zakasnitev drzanja oddaje po koncu CW sporocila, 2/10 s, fiksna vrednost, da se ne sprozi "roger beep"
	movwf	tx_tail_cntr		; Stevec zadrzanja TX po zadnji detekciji zahteve za oddajo
	bsf	flag_0,ptt_flg		; Postavim zastavico zahteve za aktiviranje PTT
	goto	zan_ptt_flg_dwn_cmn

zan_ptt_flg_dwn_s2
	movlw	tx_tail_del		; Zakasnitev drzanja oddaje po koncu aktivnega sprejema
	movwf	tx_tail_cntr		; Stevec zadrzanja TX po zadnji detekciji zahteve za oddajo
	bsf	flag_0,ptt_flg		; Postavim zastavico zahteve za aktiviranje PTT
	goto	zan_ptt_flg_dwn_cmn

zan_ptt_flg_dwn_s3
	movlw	d'5'			; Zakasnitev drzanja oddaje po koncu aktivnega sprejema na kanalu 6m crossconnect
	movwf	tx_tail_cntr		; Stevec zadrzanja TX po zadnji detekciji zahteve za oddajo
	bsf	flag_0,ptt_flg		; Postavim zastavico zahteve za aktiviranje PTT
	goto	zan_ptt_flg_dwn_cmn

zan_ptt_flg_dwn_cmn
	bsf	flag_0,ptt_flg		; Skupni del kode zahteve za akiviranje PTT, postavim zastavico zahteve za aktiviranje PTT

zan_ptt_flg_dwn_ok
	goto	zan_ptt_flg_ok

zan_ptt_flg_act
	btfsc	flag_0,cw_msg_ptt	; Zahteva za prozenje oddaje - CW sporocilo, ce ni, preskoci
	goto	zan_ptt_flg_act_s1	; Da, ukrepaj
	btfsc	flag_0,rec		; Detekcija aktivnega sprejema, ce ni, preskoci
	goto	zan_ptt_flg_act_s2	; Da, ukrepaj
	btfsc	flag_1,xcnct		; Detekcija aktivnega sprejema na 6m crossconect RX, ce ni, preskoci
	goto	zan_ptt_flg_act_s3	; Da, ukrepaj
	goto	zan_ptt_flg_act_ok	; Ni aktivnih zahtev za oddajo

zan_ptt_flg_act_s1
	movlw	d'2'			; Zakasnitev drzanja oddaje po koncu CW sporocila, 2/10 s, fiksna vrednost, da se ne sprozi "roger beep"
	movwf	tx_tail_cntr		; Stevec zadrzanja TX po zadnji detekciji zahteve za oddajo
	goto	zan_ptt_flg_act_ok

zan_ptt_flg_act_s2
	movlw	tx_tail_del		; Zakasnitev drzanja oddaje po koncu aktivnega sprejema
	movwf	tx_tail_cntr		; Stevec zadrzanja TX po zadnji detekciji zahteve za oddajo
	goto	zan_ptt_flg_act_ok

zan_ptt_flg_act_s3
	movf	tx_tail_cntr,w		; Stevec zadrzanja TX po zadnji detekciji zahteve za oddajo
	sublw	d'5'			; 5 - stanje stevca, ce pride do prenosa (C = 0), je stanje stevca vecje od 5, zato ga se ne popravljam
	btfss	status,carry		; Prenosa ni, stevec je manjsi od 5, preskoci popravim stevec
	goto	zan_ptt_flg_act_ok	; Stanja stevca ne popravljam
	movlw	d'5'			; Zakasnitev drzanja oddaje po koncu aktivnega sprejema na 6m xconnect kanalu, fiksna vrednost, da se ne sprozi "roger beep"
	movwf	tx_tail_cntr		; Stevec zadrzanja TX po zadnji detekciji zahteve za oddajo
	goto	zan_ptt_flg_act_ok

zan_ptt_flg_act_ok
	movf	tx_tail_cntr,f		; Stevec drzanja TX
	btfsc	status,zflag		; Stevec se ni dol-prestel do nicle, preskoci naslednji ukaz
	bcf	flag_0,ptt_flg		; Brisem zastavico zahteve za aktiviranje PTT

zan_ptt_flg_ok

;--------------------------------------------------------------------
;*      Prozenje PTT                                                *
;--------------------------------------------------------------------

;--------------------------------------------------------------------
;*      Koda za blokado RPT TX po preteku casovne omejitve          *
;--------------------------------------------------------------------
	movf	tmo_blk_tmr,f		; Stevec blokade po preteku casovne omejitve oddaje
	btfsc	status,zflag		; Stevec aktivno odsteva, repetitor blokiran, preskoci
	goto	rpt_tx_blk_ok		; Repetitor v normalnem delovanju

	movf	tmo_blk_tmr,w
	andlw	b'11111100'		; Najnizja bita ignoriram
	btfss	status,zflag		; Ce vrednost stevca 3 ali manj, ugasam TX..
	goto	zan_ptt_ok		; ..sicer z ugasanjem pocakam, da se odda CW sporocilo o izteku casovne omkejitve

	bcf	portb,ptt		; Ker je repetitor blokiran, cas do izteka blokade pa se ni pretekel, neprenehoma deaktiviram oddajo
	movlw	d'3'			; Vrednost za reinicializacijo stevca brezpogojne blokade
	btfsc	flag_0,ptt_flg		; Ce na vhodu RX ni vec signala, stevca ne reinicializiram
	movwf	tmo_blk_tmr		; Ce na vhodu RX se vedno prisoten signal, stevec reinicializiram ter vzdrzujem blokado
	goto	zan_ptt_ok		; Preostalega dela kode za prozenje PTT ne izvrsujem

rpt_tx_blk_ok
	btfsc	flag_0,ptt_flg		; Ce zahteva za oddajo ni aktivirana, preskoci
	goto	zan_ptt_xmt		; Zahteva za oddajo aktivirana

;--------------------------------------------------------------------
;*      Prozenje PTT - zahteva za oddajo ni aktivirana              *
;--------------------------------------------------------------------
	bcf	portb,ptt		; Deaktiviram oddajo, v odsotnosti zahteve za oddajo to v zanki neprenehoma ponavljam
	clrf	tx_tmout_cntr		; Brisem stevec casovne omejitve oddaje
	goto	zan_ptt_ok

;--------------------------------------------------------------------
;*      Prozenje PTT - zahteva za oddajo aktivirana                 *
;--------------------------------------------------------------------
zan_ptt_xmt
	btfsc	flag_0,ptt_flg_old	; Ce predhodna vrednost zastavice spuscena, preskoci
	goto	zan_ptt_xmt_ok		; Ni detektiran pozitivni odvod stanja zastavice PTT, ne ukrepam

	bsf	portb,ptt		; Zaznan pozitivni odvod zastavice stanja PTT, aktiviram PTT
	movlw	tx_timeout		; Cas do blokade oddaje pri neprekinjeni oddaji..
	movwf	tx_tmout_cntr		; ..dam v stevec time-out oddaje
	
zan_ptt_xmt_ok
	goto	zan_ptt_ok		; Ni detektiran pozitivni odvod stanja zastavice PTT, ne ukrepam

;---------------------------------------------------------------------
;*      Konec modula - prozenje PTT                                 *
;---------------------------------------------------------------------
zan_ptt_ok

;---------------------------------------------------------------------
;*      Filtriranje sql za detekcijo aktivnega /neakt. sprejema 70cm *
;*      .. hkrati prozenje timerjev za detekcijo LOS na vhodu primar.*
;*         RX ter prozenje roger BEEP CW "E"                         *
;---------------------------------------------------------------------
	btfss	porta,sql_det		; SQL izhod je v stanju aktivnega sprejema, preskoci
	goto	sql_inact		; SQL izhod ni v aktivnem stanju

sql_actv
	clrf	rec_off_cnt		; Stevec odcitkov naktivnega sql resetiram
	movf	rec_on_cnt,w		; Stevec odcitkov aktivnega sql za detektirani sprejem
	sublw	rec_on_dly		; Stevilo odcitkov, potrebnih, da je aktiven sprejem zaznan
	btfss	status,zflag		; Stevilo zahtevanih odcitkov dosezeno, preskoci in oznaci aktiven sprejem
	goto	sql_actv_s1		; Stevilo zahtevanih odcitkov ni dosezeno
					; *** najprej ukazi, ki se izvrsijo ob vsakem prehodu skozi zanko, vse dokler je sprejem aktiven
	movf	iden_rec_blk_del,w	; Trajanje (zakasnitev) blokade oddaje identifikacije zaradi aktivnega sprejema
	movwf	iden_rec_blk_to		; Inicializiram stevec blokade oddaje identifikacije zaradi aktivnega sprejema

;	* * *   Dodano v V1.4	
	movf	iden_no_sig_to,w	; * * * Dodano: 7.10.2021 - Stevec oddaje identifikacije zaradi odsotnosti aktivnosti RPT pred PTT
	btfsc	status,zflag		; * * * Dodano: 7.10.2021 - Stevec ni odstel do nicle, preskoci..
	bsf	flag_0,iden_tout	; * * * Dodano: 7.10.2021 - Postavi zastavico za oddajo ID sporocila
	movlw	iden_no_sig_dly		; * * * Dodano 7.10.2021 -  Trajanje (zakasnitev) do oddaje identifikacije zaradi odsotnosti aktivnosti RPT pred PTT
	movwf	iden_no_sig_to		; * * * Dodano 7.10.2021 -  Inicializiram stevec oddaje identifikacije zaradi odsotnosti aktivnosti RPT pred PTT
;	* * *   Konec kode, dodane v V1.4

	btfsc	flag_0,rec		; Ce aktivni sprejem se ni oznacen, preskoci in oznaci..
	goto	sql_actv_s0		; ..sicer za tokrat konec akcij

; *** ukazi, ki se izvrsijo le ob nastopu aktivnega sprejema
	bsf	flag_0,rec		; Oznacim novo stanje detekcije aktivnega sprejema
	movf	los_rx_tmr,w		; Stevec za meritev trajanaj izgube signala
	btfsc	status,zflag		; ce stevec se ni odstel do 0, je izguba signala trajala manj od maks. vrednosti veljavne detekcije
	goto	sql_actv_s0		; Prekinitev daljsa od maks. vrednosti, ni detekcije LOS
	movlw	los_det_vld		; Cas trajanja veljavnosti detekcije LOS..
	movwf	los_beep_tmr		; .. inicializira timer veljavnosti stanja LOS za prozenje roger beep CW "E"
sql_actv_s0
	goto	sql_actv_s2
sql_actv_s1
	incf	rec_on_cnt,f		; Povecam stevec odcitkov aktivnega sql
sql_actv_s2
	goto	sql_ok

sql_inact
	clrf	rec_on_cnt		; Stevec odcitkov aktivnega sql resetiram
	movf	rec_off_cnt,w		; Stevec odcitkov neaktivnega sql za detekcijo konca sprejema
	sublw	rec_off_dly		; Stevilo odcitkov, potrebnih, da je konec sprejem zaznan
	btfss	status,zflag		; Stevilo zahtevanih odcitkov dosezeno, preskoci in oznaci neaktiven sprejem
	goto	sql_inact_s1		; Stevilo zahtevanih odcitkov ni dosezeno
					; *** najprej ukazi, ki se izvrsijo ob vsakem prehodu skozi zanko, vse dokler je zaznan neaktiven sprejem
	btfss	flag_0,rec		; Ce neaktivni sprejem se ni oznacen, preskoci in oznaci..
	goto	sql_inactv_s0		; ..sicer za tokrat konec akcij
					; *** ukazi, ki se izvrsijo le ob koncu aktivnega sprejema
	bcf	flag_0,rec		; Oznacim detekcijo konca aktivnega sprejema
	movlw	los_det_t_maks		; Maksimalna dolzina trajanja prekinitve, da jo se obravnavam kot LOS na vhodu primarnega RX ..
	movwf	los_rx_tmr		; .. dam v stevec za meritev trajanaj izgube signala
	
sql_inactv_s0
	goto	sql_inact_s2
sql_inact_s1
	incf	rec_off_cnt,f		; Povecam stevec odcitkov neaktivnega sql
sql_inact_s2
	goto	sql_ok
sql_ok

;----------------------------------------------------------------------------
;*      Filtriranje sql za detekcijo aktivnega /neakt. sprejema crossconect *
;----------------------------------------------------------------------------
	btfsc	portb,jp1		; Vhod za detekcijo crossconect SQL je v stanju aktivnega sprejema --> preskoci
	goto	xcnct_inact		; xcnct sql det. vhod ni v aktivnem stanju

xcnct_actv
	clrf	xcnct_off_cnt		; Stevec odcitkov naktivnega xcnct sql. det. resetiram
	movf	xcnct_on_cnt,w		; Stevec odcitkov aktivnega xcnct sql. det. za detektirani sprejem
	sublw	xcnct_on_dly		; Stevilo odcitkov, potrebnih, da je aktiven sprejem zaznan
	btfss	status,zflag		; Stevilo zahtevanih odcitkov dosezeno, preskoci in oznaci aktiven sprejem
	goto	xcnct_actv_s1		; Stevilo zahtevanih odcitkov ni dosezeno

	bsf	flag_1,xcnct_sql	; Oznacim novo stanje detekcije aktivnega xcnct sprejema
	movf	iden_rec_blk_del,w	; Trajanje (zakasnitev) blokade oddaje identifikacije zaradi aktivnega sprejema
	movwf	iden_rec_blk_to		; Inicializiram stevec blokade oddaje identifikacije zaradi aktivnega sprejema

	goto	xcnct_actv_s2
xcnct_actv_s1
	incf	xcnct_on_cnt,f		; Povecam stevec odcitkov aktivnega xcnct sql. det.
xcnct_actv_s2
	goto	xcnct_ok

xcnct_inact
	clrf	xcnct_on_cnt		; Stevec odcitkov aktivnega xcnct sql. det. resetiram
	movf	xcnct_off_cnt,w		; Stevec odcitkov neaktivnega xcnct sql. det. za detekcijo konca sprejema
	sublw	xcnct_off_dly		; Stevilo odcitkov, potrebnih, da je konec sprejema crossconect zaznan
	btfss	status,zflag		; Stevilo zahtevanih odcitkov dosezeno, preskoci in oznaci neaktiven sprejem
	goto	xcnct_inact_s1		; Stevilo zahtevanih odcitkov ni dosezeno
	bcf	flag_1,xcnct_sql	; Oznacim detekcijo konca aktivnega sprejema crossconect
	goto	xcnct_inact_s2
xcnct_inact_s1
	incf	xcnct_off_cnt,f		; Povecam stevec odcitkov neaktivnega sql
xcnct_inact_s2
	goto	xcnct_ok
xcnct_ok

;----------------------------------------------------------------------------
;*      Blokada 6m --> 70cm  crossconect repetiranja                        *
;----------------------------------------------------------------------------
	btfss	flag_1,xcnct_sql	; Detektiran signal na 6m xconnect RX? Ce da, preskoci.
	goto	xcnct_blck_noxcrx	; Ne.
	movf	xcnct_blck_cnt,f	; Stevec vzdrzevanja blokade crossconnect povezave
	btfss	status,zflag		; Stevec ima vrednost nic, blokada ni aktivna, preskoci naslednji ukaz
	goto	xcnct_blck_goingon	; Blokada zaradi predolgega neprekinjenega sprejema na xcnct vhodu v teku
	bsf	flag_1,xcnct		; Dovoljena je zahteva za PTT 70cm TX zaradi aktivnega sprejema crossconect
	goto	xcnct_blck_ok

xcnct_blck_goingon
	movf	xcnct_blck_cnt,w	; Stevec vzdrzevanja blokade crossconnect povezave ..
	sublw	d'1'			; .. 1 - stevec
	btfss	status,zflag		; Ce 1s do konca blokade (ni prenosa), signal na vhodu pa se vedno prisoten, preskoci in vzdrzuj blokado
	goto	xcnct_blck_noxcrx
	incf	xcnct_blck_cnt,f	; Stevec povecam za 1 in ga drzim na konstantni vrednosti vse dokler je prisoten signal na vhodu

xcnct_blck_noxcrx
	bcf	flag_1,xcnct		; NI zahteve za PTT 70cm TX zaradi aktivnega sprejema crossconect ali aktivna blokada v teku
	movlw	xcnct_blck_tmo		; Cas do blokade zaradi predolgega sprejema na 6m xcnct vhodu
	movwf	xcnct_onair_cnt		; Stevec, ki odsteva dovoljeni cas trajanja aktivnega xconnect repetiranaj

xcnct_blck_ok


;--------------------------------------------------------------------
;*      Prozenje casovno odvisnih akcij (1/10 s, 1s, 1 min)         *
;--------------------------------------------------------------------
;--------------------------------------------------------------------
;*      Akcije prozene vsako 1/10 s                                 *
;--------------------------------------------------------------------
	decfsz	ss_cnt,f		; Stevec prehodov skozi zanko zmanjsam za 1, ce prestel do 0, cas za proznje akcij 1/10 s
	goto	zan_tmng_ok		; Ni cas za casovno odvisne akcije
	movlw	d'10'
	movwf	ss_cnt			; Stevec za prozenje akcij 1/10 s ponovno inicializiram

	movf	iden_rec_blk_to,f	; Stevec blokade oddaje identifikacije zaradi aktivnega sprejema
	btfss	status,zflag		; Stevec ima vrednost 0, dekrementiranje ni potrebno, preskoci
	decf	iden_rec_blk_to,f	; Stevec blokade oddaje identifikacije zmanjsam za 1

	movf	tx_tail_cntr,f		; Stevec zadrzanja TX po zadnji detekciji zahteve za oddajo
	btfss	status,zflag		; Stevec ima vrednost 0, dekrementiranje ni potrebno, preskoci
	decf	tx_tail_cntr,f		; Stevec zadrzanja TX po zadnji detekciji zahteve za oddajo zmanjsam za 1

	movf	tx_tail_cntr,w		; Stevec casa do padca TX repetitorja..
	sublw	tm_to_beep		; ..primerjam s konstanto, ki definira trenutek oddaje "roger beep"
	btfsc	status,zflag		; Ce vrednosti nista enaki, ne ukrepam
	bsf	flag_0,rgr_beep		; Postavim zastavico za prozenje "roger beep"

;--------------------------------------------------------------------
;*      Odstevanje stevca detekcije in trajanja LOS na vhodu RX     *
;--------------------------------------------------------------------
	movf	los_rx_tmr,w		; Stevec meritve trajanja prekinitve signala na vhodu RX za detekcijo LOS
	btfsc	status,zflag		; Ce stevec nima vrednosti 0, preskocim naslednji ukaz in ukrepam
	goto	zan_los_rx_tmr_ok	; Stevec ze odstel do nicle, ne ukrepam
	decf	los_rx_tmr,f		; Stevec zmanjsam za 1
zan_los_rx_tmr_ok

	movf	los_beep_tmr,w		; Stevec, ki odsteva cas trajanja veljavne detekcije LOS
	btfsc	status,zflag		; Ce stevec nima vrednosti 0, preskocim naslednji ukaz in ukrepam
	goto	zan_los_beep_tmr_ok	; Stevec ze odstel do nicle, ne ukrepam
	decf	los_beep_tmr,f		; Stevec zmanjsam za 1
zan_los_beep_tmr_ok

;--------------------------------------------------------------------
;*      Akcije prozene 1x / sekundo                                 *
;--------------------------------------------------------------------
	decfsz	ds_cnt,f		; Stevec desetink s zmanjsam za 1, ce prestel do 0, cas za proznje sekundnih akcij
	goto	zan_tmng_ok		; Ni se cas za sekundne akcije
	movlw	d'10'
	movwf	ds_cnt			; Stevec za prozenje sekundnih akcij s ponovno inicializiram

	movf	tmo_blk_tmr,f		; Stevec blokade po preteku casovne omejitve oddaje
	btfsc	status,zflag		; Stevec nima vrednosti 0, v teku aktivna blokada, zmanjsaj vrednost stevca
	goto	zan_tmo_blk_dec_ok	; Vrednost stevca enaka 0, s stevcem se vec ne ukvarjam

	decf	tmo_blk_tmr,f		; Zmanjsam stevec blokade po preteku casovne omejitve oddaje
	movf	tmo_blk_tmr,f		; Ponovno preverim stevec blokade po preteku casovne omejitve oddaje
	btfss	status,zflag		; Stevec dol-prestel do 0, konec blokade, ukrepaj
	goto	zan_tmo_blk_dec_ok	; Stevec se ni dol-prestel do 0, s stevcem se vec ne ukvarjam
zan_tmo_blk_reset
	bcf		portb,ptt	; Deaktiviram oddajo
	goto	zan_tmo_blk_reset	; V zanki pocakam na reset WDT
zan_tmo_blk_dec_ok

	movlw	d'24'			; Predpostavim SW1 = off, cas blokade oddaje identifikacije po aktivnem sprejemu = (49/10 + 10/100)s = 2,5s
	btfss	porta,dip_sw_1		; Ce SW1 dejansko ni sklenjen, preskocim naslednji ukaz
	movlw	d'49'				; SW1 sklenjen, cas blokade oddaje identifikacije po aktivnem sprejemu = (149/10 + 10/100)s = 5s
	movwf	iden_rec_blk_del	; Spremenljivka z vrednostjo zakasnitve prozenja odddaje CW identifikacije po koncu aktivnega sprejema

	movf	porta,w			; Stanje port A
	andlw	b'00001100'		; Izlocim stanje kodirnega stikalca 3 in 4
	movwf	uni_0			; Zacasno shranim
	bcf	status,carry
	rrf	uni_0,f			; Premaknem v desno 1 x
	bcf	status,carry
	rrf	uni_0,f			; Premaknem v desno drugic

	movf	uni_0,f
	btfsc	status,zflag		; SW3=on in SW4=on? Ce ni, preskoci
	goto	zan_sw34_onon		; SW3=on in SW4=on, ukrepaj

	decf	uni_0,f
	btfsc	status,zflag		; SW3=off in SW4=on? Ce ni, preskoci
	goto	zan_sw34_offon		; SW3=off in SW4=on, ukrepaj

	decf	uni_0,f
	btfsc	status,zflag		; SW3=on in SW4=off? Ce ni, preskoci
	goto	zan_sw34_onoff		; SW3=on in SW4=off, ukrepaj

	movlw	d'132'			; Obe stikalci sta na off, perioda oddaje identifikacije = 132s + 256 x 3s = 900s = 15 min
	movwf	tx_iden_per_l
	movlw	d'3'
	movwf	tx_iden_per_h
	goto	zan_sw34_ok

zan_sw34_onoff
	movlw	d'176'			; SW3 = on, perioda oddaje identifikacije = 176s + 256 x 4s = 1200s = 20 min
	movwf	tx_iden_per_l
	movlw	d'4'
	movwf	tx_iden_per_h
	goto	zan_sw34_ok

zan_sw34_offon
	movlw	d'8'			; SW4 = on, perioda oddaje identifikacije = 8s + 256 x 7s = 1800s = 30 min
	movwf	tx_iden_per_l
	movlw	d'7'
	movwf	tx_iden_per_h
	goto	zan_sw34_ok

zan_sw34_onon
	movlw	d'16'			; Obe stikalci sta na on, perioda oddaje identifikacije = 132s + 256 x 14s = 3600s = 60 min
	movwf	tx_iden_per_l
	movlw	d'14'
	movwf	tx_iden_per_h
	goto	zan_sw34_ok

zan_sw34_ok

	movf	iden_per_cnt_l,f	; Nizji byte stevca periode oddaje identifikacijskega sporocila
	btfss	status,zflag		; Nizji byte enak nic, preveri se visji byte
	goto	zan_iden_per_cnt_nz	; Stevec periode ni enak nic, ukrepaj
	movf	iden_per_cnt_h,f	; Visji byte stevca periode oddaje identifikacijskega sporocila
	btfss	status,zflag		; Tudi visji byte stevca enak nic, postavi zastavico
	goto	zan_iden_per_cnt_nz	; Visji byte ni enak nic, ukrepaj

	bsf	flag_0,iden_tout	; Postavim zastavico, da se je cas do oddaje novega sporocila iztekel
	goto	zan_iden_per_cnt_ok	; Za tokrat konec akcije

zan_iden_per_cnt_nz
	decf	iden_per_cnt_l,f	; Nizji byte stevca periode oddaje identifikacijskega sporocila zmanjsam za 1
	movf	iden_per_cnt_l,w	; Novo stanje nizjega bytea stevca...
	sublw	0xFF			; ...primerjam z 255
	btfsc	status,zflag		; Nizji byte ni obrnil iz 0 na 255, preskoci
	decf	iden_per_cnt_h,f	; Nizji byte obrnil iz 0 na 255, zmanjsaj se vrednost visjega bytea
zan_iden_per_cnt_ok

;--------------------------------------------------------------------
;*      Odstevanje stevca trajanja aktivnega xconnect repetiranja.  *
;--------------------------------------------------------------------
	movf	xcnct_onair_cnt,w	; Stevec, ki odsteva dovoljeni cas trajanja aktivnega xconnect repetiranja
	btfsc	status,zflag		; Ce stevec nima vrednosti 0, preskocim naslednji ukaz in ukrepam
	goto	zan_onair_cnt_ok	; Stevec ze odstel do nicle, ne ukrepam
	decfsz	xcnct_onair_cnt,f	; Stevec zmanjsam za 1, ce prestel do 0, preskocim naslednji ukaz in ukrepam
	goto	zan_onair_cnt_ok
	movlw	xcnct_blck_durat	; Trajanje blokade zaradi predolgega neprekinjenega sprejema na 6m xcnct vhodu (T = n x 1s, priporocena vrednost 60)
	movwf	xcnct_blck_cnt		; Inicializiram stevec vzdrzevanja blokade PTT zaradi sprejema na crossconnect povezavi 6m vhoda
zan_onair_cnt_ok

;--------------------------------------------------------------------
;*      Odstevanje stevca trajanja blokade xconnect repetiranja.    *
;--------------------------------------------------------------------
	movf	xcnct_blck_cnt,w	; Stevec, ki odsteva cas trajanja blokade xconnect repetiranja
	btfsc	status,zflag		; Ce stevec nima vrednosti 0, preskocim naslednji ukaz in ukrepam
	goto	zan_blck_cnt_ok		; Stevec ze odstel do nicle, ne ukrepam
	decf	xcnct_blck_cnt,f	; Stevec zmanjsam za 1
zan_blck_cnt_ok

;---------------------------------------------------------------------
;*      Odstevanje stevca do oddaje identifikacije zaradi odsotnosti *
;*      aktivnosti RPT pred PTT.                                     *
;*      Dodano v Ver. 1.4, 7.10.2021                                 *
;---------------------------------------------------------------------
	movf	iden_no_sig_to,w	; Stevec, ki odsteva cas do oddaje identifikacije zaradi odsotnosti aktivnosti RPT pred PTT
	btfsc	status,zflag		; Ce stevec nima vrednosti 0, preskoci naslednji ukaz in ukrepaj
	goto	zan_idtx_noact_cnt_ok	; Stevec ze odstel do nicle, ne ukrepam
	decf	iden_no_sig_to,f	; Stevec --
zan_idtx_noact_cnt_ok

;--------------------------------------------------------------------
;*      Akcije prozene 1x / minuto                                  *
;--------------------------------------------------------------------
	decfsz	sec_cnt,f		; Sekundni stevec zmanjsam za 1, ce prestel do 0, cas za proznje sekundnih akcij
	goto	zan_tmng_ok		; Ni se cas za sekundne akcije
	movlw	d'60'
	movwf	sec_cnt			; Stevec za prozenje sekundnih akcij s ponovno inicializiram

	movf	tx_tmout_cntr,f		; Stevec casovne omejitve oddaje
	btfsc	status,zflag		; Stevec nima vrednosti 0, oddaja v teku, casovna omejitev oddaje se se ni iztekla
	goto	zan_tx_tmout_ok		; Vrednost stevca enaka 0, s stevcem se ne ukvarjam

	decfsz	tx_tmout_cntr,f		; Zmanjsam stevec casovne omejitve oddaje
	goto	zan_tx_tmout_ok		; Stevec se ni dol-prestel do 0, s stevcem se tokrat vec ne ukvarjam

	movlw	d'15'			; Cas trajanja brezpogojne blokade oddaje (3s + dolzina tipkanja CW sporocila o preteku casovne omejitve TX)..
	movwf	tmo_blk_tmr		; ..dam v stevec trajanja brezpogojne blokade TX
	movlw	d'1'			; Relativni naslov prvega znaka sporocila o preteku casovne omejitve oddaje
	movwf	msg_iden_pnt		; ..dam v kazalec oddaje

zan_tx_tmout_ok

;--------------------------------------------------------------------
;*      Konec kode s casovno odvisnimi akcijami                     *
;--------------------------------------------------------------------
zan_tmng_ok

;--------------------------------------------------------------------
;*      Prozenje in oddaja "roger beep" tona                        *
;--------------------------------------------------------------------
	movf	rgr_beep_cntr,f		; Preverim stevec trajaja oddaje "roger beep" tona
	btfss	status,zflag		; Ce stevec vrednost 0, preskoci in preveri, ali so pogoji za prozenje oddaje "roger beep" tona
	goto	zan_rgr_beep_s0		; Oddja "roger beep" tona v teku, zmanjsaj stevec

	btfss	flag_0,rgr_beep		; Ce zastavica za prozenje "roger beep" postavljena, preskoci..
	goto	zan_rgr_beep_ok		; ..sicer tokrat nic prozenja oddaje "roger beep" tona

	bcf	flag_0,rgr_beep		; Brisem zastavico za prozenje "roger beep"

	movf	los_beep_tmr,w		; Stevec veljavnosti detekcije LOS
	btfss	status,zflag		; Ni zahteve za roger beep "LOS", oddam obicajni roger beep
	goto	zan_rgr_beep_s01
	movlw	d'47'			; Daljsi roger beep (= CW "B") za signalizacijo napajanja z baterije ... 47 (31)
	btfss	portb,jp2		; Ce stanje na JP2 vhodu visoko, omrezno napajanje ni prisotno, preskocim naslednji ukaz
	movlw	d'16'			; Krajsi roger beep (= CW "I") kadar je omrezno napajanje prisotno
	goto	zan_rgr_beep_s02
zan_rgr_beep_s01
	movlw	d'5'			; Roger beep (= CW "E") za javljanje LOS na vhodu RX, ce aktiven, povozi vse ostale vrste roger beep
zan_rgr_beep_s02
	movwf	rgr_beep_cntr 		; ..dam v stevec za generiranje CW tona
	bsf	flag_0,cw_tone		; Postavim zastavico generiranja CW tona
	goto	zan_rgr_beep_ok

zan_rgr_beep_s0
	decfsz	rgr_beep_cntr,f		; Zmanjsam stevec trajanja "roger beep" tona in ce presteje do nic, ton dokoncno ugasnem
	goto	zan_rgr_beep_s1		; Vmesno ugasanje in priziganje tona za realizacijo razlicnih "roger beep"
	bcf	flag_0,cw_tone		; Spustim zastavico generiranja CW tona
	goto	zan_rgr_beep_ok

zan_rgr_beep_s1
	movf	rgr_beep_cntr,w		; Stevec trajanja "roger beep" tona ..
	sublw	d'0'				; ..primerjam z 0
	btfsc	status,zflag		; Ce ni enako preskocim in preverjam dalje ..
	goto	zan_rgr_beep_off	; ..sicer grem na del kode, ki ugasne CW ton

	movf	rgr_beep_cntr,w		; Stevec trajanja "roger beep" tona ..   <==== zacetek oddaje roger beep "E"
	sublw	d'5'				; ..primerjam s 5 (3)
	btfsc	status,zflag		; Ce ni enako preskocim in preverjam dalje ..
	goto	zan_rgr_beep_on		; ..sicer grem na del kode, ki vkljuci CW ton

	movf	rgr_beep_cntr,w		; Stevec trajanja "roger beep" tona ..
	sublw	d'11'				; ..primerjam z 11 (7)
	btfsc	status,zflag		; Ce ni enako preskocim in preverjam dalje ..
	goto	zan_rgr_beep_off	; ..sicer grem na del kode, ki ugasne CW ton

	movf	rgr_beep_cntr,w		; Stevec trajanja "roger beep" tona ..   <==== zacetek oddaje roger beep "I"
	sublw	d'16'				; ..primerjam z 16 (10)
	btfsc	status,zflag		; Ce ni enako preskocim in preverjam dalje ..
	goto	zan_rgr_beep_on		; ..sicer grem na del kode, ki vkljuci CW ton

	movf	rgr_beep_cntr,w		; Stevec trajanja "roger beep" tona ..
	sublw	d'20'				; ..primerjam z 20 (14)
	btfsc	status,zflag		; Ce ni enako preskocim in preverjam dalje ..
	goto	zan_rgr_beep_off	; ..sicer grem na del kode, ki ugasne CW ton

	movf	rgr_beep_cntr,w		; Stevec trajanja "roger beep" tona ..
	sublw	d'25'				; ..primerjam s 25 (17)
	btfsc	status,zflag		; Ce ni enako preskocim in preverjam dalje ..
	goto	zan_rgr_beep_on		; ..sicer grem na del kode, ki vkljuci CW ton

	movf	rgr_beep_cntr,w		; Stevec trajanja "roger beep" tona ..
	sublw	d'31'				; ..primerjam z 31 (21)
	btfsc	status,zflag		; Ce ni enako preskocim in preverjam dalje ..
	goto	zan_rgr_beep_off	; ..sicer grem na del kode, ki ugasne CW ton

	goto	zan_rgr_beep_ok		; Tokrat nic vmesne akcije

zan_rgr_beep_on
	bsf	flag_0,cw_tone		; Dvignem zastavico generiranja CW tona
	goto	zan_rgr_beep_ok		

zan_rgr_beep_off
	bcf	flag_0,cw_tone		; Spustim zastavico generiranja CW tona
	goto	zan_rgr_beep_ok

zan_rgr_beep_ok

;--------------------------------------------------------------------
;*      Prozenje oddaje CW identifikacijskega sporocila             *
;--------------------------------------------------------------------
	movf	tmo_blk_tmr,f		; Stevec blokade po preteku casovne omejitve oddaje
	btfss	status,zflag		; Blokada TX ni aktivirana, preskoci, preskoci..
	goto	zan_odl_tmr_ok		; ..sicer ni pogojev za oddajo identifikacijskega sporocila (aktivirana blokada casovne omejitve TX)

	btfss	flag_0,iden_tout	; Zastavica, da se je cas do oddaje novega sporocila iztekel, postavljena? Da, preskoci in preverjaj dalje.
	goto	zan_odl_tmr_ok		; Ne, ni pogojev za oddajo identifikacijskega sporocila

	movf	iden_rec_blk_to,f	; Detektiran aktivni sprejem (stevec blokade nima vrednosti nic)?
	btfss	status,zflag		; Ce ne, so pogoji za oddajo identifikacijskega sporocila izpolnjeni
	goto	zan_odl_tmr_ok		; Ce da, ni pogojev za oddajo identifikacijskega sporocila

	bcf	flag_0,iden_tout	; Brisem zastavico, da se je cas do oddaje novega sporocila iztekel.
	bsf	flag_0,tx_iden_msg	; Postavim zastavico zahteve za takojsnjo oddajo identifikacijskega sporocila

	movf	tx_iden_per_l,w		; Perioda oddaje sporocila v timersko prozenem rezimu, nizji byte
	movwf	iden_per_cnt_l		; Ponovno inicializiram stevec periode oddaje identifikacijskega sporocila, nizji byte
	movf	tx_iden_per_h,w		; Perioda oddaje sporocila v timersko prozenem rezimu, visji byte
	movwf	iden_per_cnt_h		; Ponovno inicializiram stevec periode oddaje identifikacijskega sporocila, visji byte
	goto	zan_odl_tmr_ok		; Konec odlocitvene kode prozenja oddaje identifikacijskega sporocila

zan_odl_tmr_ok

;--------------------------------------------------------------------
;*   Oddaja CW sporocila                                            *
;--------------------------------------------------------------------
cdc_eot_tx
	movf	msg_iden_pnt,f		; Kazalec oddaje identifikacijskega sporocila
	btfss	status,zflag		; Kazalec ima vrednost nic, oddaja iden. sporocila ni v teku
	goto	cdc_eot_tx_s0		; Oddaja CW sporocila v teku
	goto	cdc_eot_tx_cwmsg	; Oddaja CW sporocila ni v teku

cdc_eot_tx_s0
	movf	cw_tone_cnt,f		; Stevec trajanja CW tona
	btfss	status,zflag		; CW ton se vec ne oddaja
	goto	cdc_eot_tx_cwtone	; Oddaja CW tona v teku

	movf	cw_paus_cnt,f		; Stevec trajanja CW pavze
	btfss	status,zflag		; CW pavze je konec
	goto	cdc_eot_tx_cwpaus	; Oddaja CW pavze v teku

	movf	dit_dah_cnt,f		; Stevec oddaje crtic pikic
	btfss	status,zflag		; Ce vrednost stevca enaka nic, potem pojdi na oddajo naslednjega znaka
	goto	cdc_eot_tx_ditdah	; ..sicer pripravi za oddajo naslednjo crtico oz. pikico

	movlw	0x00			; Ocistim W
	call	iden_msg_tab		; Iz tabele preberem stevilo znakov sporocila..
	movwf	fsr			; ..zacasno shranim
	movf	fsr,w
	movf	msg_iden_pnt,w		; Kazalec_oddajanega_znaka
	subwf	fsr,w			; stevilo_znakov_sporocila -  kazalec_oddajanega_znaka
	btfss	status,carry		; Ce ni prenosa, sporocilo se ni oddano, preskoci
	goto	cdc_eot_tx_end		; Konec oddaje

	movf	msg_iden_pnt,w		; Kazalec na znak, ki gre naslednji na oddajo
	call	iden_msg_tab		; Iz tabele preberem znak..
	movwf	fsr			; ..in ga zacasno shranim

	movf	fsr,f			; Precitani znak
	btfsc	status,zflag		; Vrednost ni enaka nic, oddaj CW znak
	goto	cdc_eot_tx_icpause	; Oddaj pavzo med dvema CW znakoma

	btfss	fsr,7			; Ce bit 7 postavljen ...
	goto	cdc_eot_tx_s1		; Ni, dolzina CW znaka zapisana s 3 biti
	btfss	fsr,6			; .. in bit 6 postavljen ...
	goto	cdc_eot_tx_s1		; Ni, dolzina CW znaka zapisana s 3 biti
	movlw	d'6'			; .. CW koda dolga 6 "bitov"

	rlf	fsr,f			; Zacasno shranjen zapis CW kode premaknem za dve mesti v levo, da pride na najvisje mesto prvi "bit"
	rlf	fsr,f
	goto	cdc_eot_tx_s2
cdc_eot_tx_s1
	swapf	fsr,w			; Zamenjam nibla..
	movwf	uni_0
	rrf	uni_0,w			; .. premaknem za eno mesto v levo..
	andlw	0x07			; .. izlocim le najnizje 3 bite in dobim stevilo crtic in pikic aktualnega CW znaka
	
	rlf	fsr,f			; Zacasno shranjen zapis CW kode premaknem za tri mesta v levo, da pride na najvisje mesto prvi "bit"
	rlf	fsr,f
	rlf	fsr,f
cdc_eot_tx_s2
	movwf	dit_dah_cnt		; Stevilo pikic in crtic znaka za oddajo
	incf	msg_iden_pnt,f		; Premaknem kazalec na naslednji znak

cdc_eot_tx_ditdah
	rlf	fsr,f			; Crtico ali pikico, ki gre na oddajo, dam v carry
	movlw	dit_lngth		; Predpostavim, da je pikica
	btfsc	status,carry		; Je pikica, preskoci
	movlw	dah_lngth		; Naslednji znak za na oddajo je crtica
	movwf	cw_tone_cnt 		; Dam v stevec za generiranje CW tona
	bsf	flag_0,cw_tone		; Postavim zastavico generiranja CW tona
	decf	dit_dah_cnt,f		; Zmanjsam stevec crtic in pikic, ki cakajo na oddajo		
	goto	cdc_eot_tx_ok
cdc_eot_tx_end
	clrf	msg_iden_pnt		; Resetiram kazalec oddaje
	bcf	flag_0,cw_msg_ptt	; Ugasnem PTT
	bcf	flag_0,tx_iden_msg	; Brisem tudi zastavico oddaje iden. sporocila, ce bi ta nastopila med samo odajo CW sporocila (zunanje prozenje)
	goto	cdc_eot_tx_ok

cdc_eot_tx_icpause
	clrf	dit_dah_cnt		; Stevilo pikic in crtic znaka za oddajo je nic
	movlw	interchar_lngth		; Oddam samo zakasnitev v trajanju dolzine presledka za zadnjo crtico ali pikico
	movwf	cw_paus_cnt		; Inicializiram stevec trajanja pavze (stanje brez tona)
	incf	msg_iden_pnt,f		; Premaknem kazalec na naslednji znak
	goto	cdc_eot_tx_ok

cdc_eot_tx_cwtone
	decfsz	cw_tone_cnt,f		; Zmanjsam stevec trajanaj CW tona. Ce prestel do nic, ukrepaj
	goto	cdc_eot_tx_cwtn_ok	; Sicer nadaljuj z oddajo CW tona
	movlw	interditdah_lngth	; Dolzina presledka med dvema crticama ali pikicama
	movf	dit_dah_cnt,f		; Stevilo crtic / pikic, ki cakajo na oddajo
	btfsc	status,zflag		; Ce ni enako nic, ok
	movlw	interchar_lngth		; Sicer zahtevam zakasnitev v trajanju dolzine presledka za zadnjo crtico ali pikico
	movwf	cw_paus_cnt		; Inicializiram stevec trajanja pavze (stanje brez tona)
	bcf	flag_0,cw_tone		; Ugasnem CW ton
cdc_eot_tx_cwtn_ok
	goto	cdc_eot_tx_ok

cdc_eot_tx_cwpaus
	decf	cw_paus_cnt,f		; Zmanjsam stevec trajanja pavze CW znakov
	goto	cdc_eot_tx_ok

;--------------------------------------------------------------------
;*   Prozenje oddaje CW sporocil                                    *
;--------------------------------------------------------------------
cdc_eot_tx_cwmsg
	btfss	flag_0,tx_iden_msg	; Ce zastavica zahteve za oddajo identifikacijskega sporocila dvignjena, preskoci in sprozi oddajo..
	goto	cdc_eot_tx_ok		; ..sicer nic akcije
	
	bcf	flag_0,tx_iden_msg	; Brisem zastavico zahteve za oddajo identifikacijskega sporocila
	movlw	d'9'			; Relativni naslov prvega znaka identifikacijskega sporocila
	movwf	msg_iden_pnt		; ..dam v kazalec oddaje
	bcf	intcon,gie		; Onemogocim prekinitve
	bcf	flag_0,cw_tone		; Brisem vse nepotrebne zastavice
	bsf	flag_0,cw_msg_ptt	; Zahtevam aktivacijo PTT
	bsf	intcon,gie		; Zopet omogocim prekinitve
	goto	cdc_eot_tx_ok

cdc_eot_tx_ok

;--------------------------------------------------------------------
;*      Konec glavne programske zanke.                              * 
;--------------------------------------------------------------------
	goto	zanka

;********************************************************************
;*                      That's all folks!                           *
;********************************************************************
	end




